﻿
#region namespace
using System;
using System.Drawing;
using Microsoft.VisualBasic;
using System.Windows.Forms; //IWin32Window
using System.IO;
using System.Collections; //ArrayList
using System.Collections.Generic; //List<>

using FunctionBay.RecurDyn.ProcessNet;
using ProcessNet;
//For C#
//using FunctionBay.RecurDyn.ProcessNet.Chart;
//using FunctionBay.RecurDyn.ProcessNet.MTT2D;
//using FunctionBay.RecurDyn.ProcessNet.FFlex;
//using FunctionBay.RecurDyn.ProcessNet.RFlex;
//using FunctionBay.RecurDyn.ProcessNet.Tire;

//For VB.NET
//Imports FunctionBay.RecurDyn.ProcessNet
//Imports FunctionBay.RecurDyn.ProcessNet.Chart
//Imports FunctionBay.RecurDyn.ProcessNet.MTT2D
//Imports FunctionBay.RecurDyn.ProcessNet.FFlex
//Imports FunctionBay.RecurDyn.ProcessNet.RFlex
#endregion

public struct SBodyData
{
    public IBody selectedBody;
    public uint bkLayer;
}
namespace ProcessNet.csproj
{
    public partial class ThisApplication
    {
        public void RegisterFunction()
        {

            IRibbonManager ribbonManager = application.RibbonManager;

            IRibbonTab ribbonTab = ribbonManager.FindRibbonTab("Customize");

            IRibbonGroup ribbonGroup = ribbonTab.AddRibbonGroup("PNet Example");



            //ID for user created processNet function must be between 8000 - 8999 .

            IMenuControl menuControl = ribbonGroup.AddMenuControl(MenuControlType.MenuControlType_Button, 8001);



            // Set example control information.           

            IntPtr iIcon = ProcessNet.csproj.Properties.Resources.example.GetHicon();

            menuControl.SetIcon(iIcon);

            menuControl.UseBigIcon = true;



            menuControl.Caption = "SelectFit";

            menuControl.Tooltip = "Fit for selected entities";

            //menuControl.Description = "MyDescription";

            menuControl.UseProcessNetFunction = true; //if ID is not between 8000 - 8999, this will set ID to 8000



            // Get current ProcessNet dll fullpath. 'ProcessNetDllPath' must be absolute path.

            string assemblyname = System.Reflection.Assembly.GetExecutingAssembly().GetName().Name;

            string ProcessNetDllPath = @"D:\Pnet\SelectFit.dll";



            menuControl.ProcessNetDllPath = ProcessNetDllPath;

            menuControl.ProcessNetFunctionName = "SelectFit";

        }

        
        public void SelectFit()
        {
            refFrame1 = modelDocument.CreateReferenceFrame();
            refFrame1.SetOrigin(100, 0, 0);

            refFrame2 = modelDocument.CreateReferenceFrame();
            refFrame2.SetOrigin(0, 200, 0);

            LayerFilterTypeEx bkLayerFilterType = modelDocument.LayerSetting.LayerFilter;
            uint[] bkVisibleLayer = modelDocument.LayerSetting.VisibleLayer;

            IGenericCollection genericCollection = modelDocument.GetSelectedEntities();

            if (null != genericCollection)
            {

                ArrayList listSelectedBody = new ArrayList();

                foreach (IGeneric selectedEntity in genericCollection)
                {
                    if (selectedEntity is IBody)
                    {
                        IBody body = (IBody)selectedEntity;

                        SBodyData sBodyData;
                        sBodyData.selectedBody = body;
                        sBodyData.bkLayer = body.LayerNumber;
                        body.LayerNumber = 254; //set the temporary layer No.


                        listSelectedBody.Add(sBodyData);
                    }
                }

                if (listSelectedBody.Count > 0)
                {
                    modelDocument.LayerSetting.LayerFilter = LayerFilterTypeEx.LayerFilterTypeEx_MultiLayer;
                    uint[] uiVisibleLayer = { 254 };
                    modelDocument.LayerSetting.VisibleLayer = uiVisibleLayer;

                    modelDocument.Fit();

                    foreach (SBodyData selectedBodyData in listSelectedBody)
                    {
                        selectedBodyData.selectedBody.LayerNumber = selectedBodyData.bkLayer;
                    }

                    modelDocument.LayerSetting.LayerFilter = bkLayerFilterType;
                    modelDocument.LayerSetting.VisibleLayer = bkVisibleLayer;
                    modelDocument.Redraw();
                }
            }
            else
            {
                application.PrintMessage("<Warning> Select more than one body in advance.");
            }

        }
        #region VSTA generated code
        private void ThisApplication_Startup(object sender, EventArgs e)
        {
            System.Threading.Thread.CurrentThread.CurrentCulture = System.Globalization.CultureInfo.InvariantCulture;
            MainWindow = new WinWrapper(System.Diagnostics.Process.GetCurrentProcess().MainWindowHandle);
        }

        private void ThisApplication_Shutdown(object sender, EventArgs e)
        {

        }

        private void InternalStartup()
        {
            this.Startup += new System.EventHandler(ThisApplication_Startup);
            this.Shutdown += new System.EventHandler(ThisApplication_Shutdown);
        }
        #endregion

        #region RecurDyn generated code

        #region Common Variables
        static public IApplication application;
        public IModelDocument modelDocument = null;
        public IPlotDocument plotDocument = null;
        public ISubSystem model = null;

        public IReferenceFrame refFrame1 = null;
        public IReferenceFrame refFrame2 = null;

        #endregion

        #region WinForms
        private System.Windows.Forms.IWin32Window MainWindow;

        //If you made your own Form, please Show with argurment, MainWindow
        //MyForm.Show(MainWindow);

        class WinWrapper : System.Windows.Forms.IWin32Window
        {
            public WinWrapper(IntPtr oHandle)
            {
                _oHwnd = oHandle;
            }

            public IntPtr Handle
            {
                get { return _oHwnd; }
            }

            private IntPtr _oHwnd;
        }

        public void CloseWindow()
        {
            System.Windows.Forms.Application.Exit();
        }
        #endregion

        #region Initialize and Dispose By RecurDyn
        [SetUp]
        public void Initialize() //Initialize() will be called automatically before ProcessNet function call.
        {
            application = RecurDynApplication as IApplication;
            modelDocument = application.ActiveModelDocument;
            plotDocument = application.ActivePlotDocument;
            if (modelDocument == null & plotDocument == null)
            {
                application.PrintMessage("No model file");
                modelDocument = application.NewModelDocument("Examples");
            }
            if (modelDocument != null)
            {
                model = modelDocument.Model;
            }
        }

        [TearDown]
        public void Dispose() //Dispose() will be called automatically after ProcessNet function call.
        {
            modelDocument = application.ActiveModelDocument;
            if (modelDocument != null)
            {
                if (modelDocument.Validate() == true)
                {
                    //Redraw() and UpdateDatabaseWindow() can take more time in a heavy model.
                    modelDocument.Redraw();
                    //modelDocument.PostProcess(); //UpdateDatabaseWindow(), SetModified();
                    modelDocument.UpdateDatabaseWindow(); //If you call SetModified(), Animation will be reset.
                    modelDocument.SetModified();
                    modelDocument.SetUndoHistory("ProcessNet");
                }
            }
        }
        #endregion

        #endregion //RecurDyn generated code

    }

    #region Assert Utility Class
    public class Assert
    {
        static public void AreEqualIgnoringCase(string expected, string actual, string message)
        {
            if (0 != String.Compare(expected, actual, true)) //ignoreCase : true
            {
                ThisApplication.application.PrintMessage(message);
            }
        }

        static public void AreEqualIgnoringCase(string expected, string actual)
        {
            String message = "Expected: " + expected + ", Actual: " + actual;
            AreEqualIgnoringCase(expected, actual, message);
        }

        static public void AreEqual(string expected, string actual, string message)
        {
            if (0 != String.Compare(expected, actual, false)) //ignoreCase : false
            {
                ThisApplication.application.PrintMessage(message);
            }
        }

        static public void AreEqual(string expected, string actual)
        {
            String message = "Expected: " + expected + ", Actual: " + actual;
            AreEqual(expected, actual, message);
        }

        static public void AreEqual(double expected, double actual, double delta, string message)
        {
            if (Math.Abs(expected - actual) > delta)
            {
                ThisApplication.application.PrintMessage(message);
            }
        }

        static public void AreEqual(double expected, double actual, double delta)
        {
            String message = "Expected: " + expected + ", Actual: " + actual;
            AreEqual(expected, actual, delta, message);
        }

        static public void AreEqual(double expected, double actual)
        {
            String message = "Expected: " + expected + ", Actual: " + actual;
            AreEqual(expected, actual, 0, message);
        }

        static public void AreEqual(bool expected, bool actual)
        {
            String message = "Expected: " + expected.ToString() + ", Actual: " + actual.ToString();
            AreEqual(expected.ToString(), actual.ToString(), message);
        }

        //static public void AreEqual(Enum expected, Enum actual)
        //{
        //    String message = "Expected: " + expected.ToString() + ", Actual: " + actual.ToString();
        //    AreEqual(expected.ToString(), actual.ToString(), message);
        //}

        static public void AreEqual(object expected, object actual)
        {
            String message = "Expected: " + expected.ToString() + ", Actual: " + actual.ToString();
            AreEqual(expected.ToString(), actual.ToString(), message);
        }

        static public void AreNotEqual(object expected, object actual)
        {
            String message = "Expected: " + expected.ToString() + ", Actual: " + actual.ToString();
            AreNotEqual(expected.ToString(), actual.ToString(), message);
        }

        static public void AreNotEqual(string expected, string actual, string message)
        {
            if (0 == String.Compare(expected, actual, false)) //ignoreCase : false
            {
                ThisApplication.application.PrintMessage(message);
            }
        }

        /// If object is not null Throw the message in output window      
        static public void IsNull(object actual)
        {
            if (actual != null)
                ThisApplication.application.PrintMessage(actual.ToString() + "is not null");
        }

        /// If object is null Throw the message in output window       
        static public void IsNotNull(object actual)
        {
            if (actual == null)
                ThisApplication.application.PrintMessage("Object is null");
        }
    }


    #endregion

    #region Attribute
    //internal use only
    [AttributeUsage(AttributeTargets.Method, AllowMultiple = false)]
    public class TestAttribute : Attribute
    {
    }

    [AttributeUsage(AttributeTargets.Method, AllowMultiple = false)]
    public class TearDownAttribute : Attribute
    {
    }

    [AttributeUsage(AttributeTargets.Method, AllowMultiple = false)]
    public class SetUpAttribute : Attribute
    {
    }

    [AttributeUsage(AttributeTargets.Method, AllowMultiple = false)]
    public class CategoryAttribute : Attribute
    {
        public CategoryAttribute(string name) { }
    }

    [AttributeUsage(AttributeTargets.Method, AllowMultiple = false)]
    public class IgnoreAttribute : Attribute
    {
        public IgnoreAttribute(string name) { }
    }
    #endregion

}
